import { VNode, Ref, AppContext, Component, ExtractPropTypes, App } from 'vue';

interface OverlayMeta {
    /** 调取失败，更改 visible，且当 animation 结束后销毁 */
    cancel: Function;
    /** 调取成功，更改 visible，且当 animation 结束后销毁 */
    confirm: Function;
    /** 销毁当前实例（立即，且调用失败），不是 overlay 则调用 reject */
    vanish: Function;
    /** vnode 当前包装层的 VNode */
    vnode?: VNode;
    /** visible 包装层属性，控制弹出层显示与隐藏 */
    visible: Ref<boolean>;
    /** 使用默认的值(component 式调用) */
    isTemplate?: boolean;
}

interface UseOverlayMetaOptions {
    /** 动画时长，避免过早销毁组件 */
    animation?: number;
    /** 是否立即将 visible 设为 true */
    immediate?: boolean;
    /**
     * template 使用的 v-model 字段
     *
     * @default 'visible'
     */
    model?: string;
}
/**
 * 获取弹出层元信息
 * @function cancel 调取失败，更改 visible，且当 animation 结束后销毁
 * @function confirm 调取成功，更改 visible，且当 animation 结束后销毁
 * @function vanish 销毁当前实例（立即调用，且会调用失败）
 * @field vnode 当前包装层的 VNode
 * @field visible 包装层属性，控制弹出层显示与隐藏
 * @returns
 */
declare function useOverlayMeta(options?: UseOverlayMetaOptions): OverlayMeta;

interface MountOverlayOptions {
    /** 渲染时挂在的 dom 节点 */
    root?: HTMLElement;
    /** 用于继承当前应用上下文 */
    appContext?: AppContext;
}

declare type ImperativePromise<T = any> = Promise<T> & {
    cancel: Function;
    confirm: Function;
};
declare type ExtractInferTypes<Props> = Props extends ExtractPropTypes<infer E> ? E : ExtractPropTypes<Props>;
declare type ImperativeOverlay<Params, Resolved> = (props?: ExtractInferTypes<Params>, options?: MountOverlayOptions) => ImperativePromise<Resolved>;
/**
 * 转换命令式弹出层 Api
 * @param component 组件
 * @param options 弹出层配置
 * @returns 命令式弹出层
 */
declare function createOverlay<Params = any, Resolved = void>(component: Component): ImperativeOverlay<Params, Resolved>;

interface ExecuteOverlayOptions<P = any> extends MountOverlayOptions {
    props?: P;
}
/**
 * 调起 overlay 组件
 * @param component 组件
 * @param props 参数
 * @param options imperativeOverlay 的配置
 * @returns
 */
declare function executeOverlay<P = any, R = any>(component: Component, options?: ExecuteOverlayOptions<P>): ImperativePromise<R>;

declare const install: (app: App) => void;
declare const unoverlay: {
    install: (app: App) => void;
};

export { ImperativeOverlay, createOverlay, unoverlay as default, executeOverlay, install, useOverlayMeta };
